"""This module allows registration and management of jobruns, jobrun actions and log files"""

try:
    from urllib.parse import urlparse
except ImportError:
    import urlparse
import requests
import dateutil.parser

CALLBACK_BASE_URL = 'http://jobcontroltest.vgt.vito.be:8090'

REST_HEADERS = {'Accept': 'application/json',
                'Content-Type': 'application/json'}

class CallbackClient(object):
    """Client allowing to register jobruns, jobrun actions and logs."""

    def __init__(self, baseurl=CALLBACK_BASE_URL):

        self.baseurl = baseurl

    def register_jobrun(self, job_id, username, start_time=None):
        """Register a jobrun"""

        data = {"job_id": job_id,
                "username": username}
        if start_time is not None:
            data["start_time"] = start_time.isoformat()

        url = urlparse.urljoin(self.baseurl, 'jobruns')
        response = requests.post(url,
                                 json=data,
                                 headers=REST_HEADERS)

        if response.status_code == requests.codes.created:
            return JobRun.build(response.json())
        else:
            response.raise_for_status()

    def jobrun_finished(self, jobrun_id, time=None):
        """Change the state of a jobrun to FINISHED"""
        return self._change_jobrun_state(jobrun_id, "FINISHED", time)

    def jobrun_failed(self, jobrun_id, time=None):
        """Change the state of a jobrun to FAILED"""
        return self._change_jobrun_state(jobrun_id, "FAILED", time)

    def jobrun_action_finished(self, jobrun_id, jobrun_action_id, time=None):
        """Change the state of a jobrun action to FINISHED"""
        return self._change_jobrun_action_state(jobrun_id, jobrun_action_id, "FINISHED", time)

    def jobrun_action_failed(self, jobrun_id, jobrun_action_id, time=None):
        """Change the state of a jobrun action to FAILED"""
        return self._change_jobrun_action_state(jobrun_id, jobrun_action_id, "FAILED", time)

    def register_jobrun_action(self, jobrun_id, name, start_time=None):
        """Register a jobrun action"""

        data = {"name": name}
        if start_time is not None:
            data["start_time"] = start_time.isoformat()

        url = self.baseurl + '/jobruns/' + jobrun_id + '/actions'
        response = requests.post(url,
                                 json=data,
                                 headers=REST_HEADERS)

        if response.status_code == requests.codes.created:
            return JobRunAction.build(response.json())
        else:
            response.raise_for_status()

    def add_jobrun_action_log(self, jobrun_id, jobrun_action_id, message):
        """Add a log entry for a jobrun action"""

        data = {"message": message}

        requests.post(self.baseurl
                      + '/jobruns/' + jobrun_id + '/actions/'
                      + jobrun_action_id + '/logs',
                      json=data,
                      headers=REST_HEADERS)

    def _change_jobrun_state(self, jobrun_id, state, time=None):

        data = {"state": state}
        if time is not None:
            data["time"] = time.isoformat()

        url = self.baseurl + '/jobruns/' + jobrun_id
        response = requests.patch(url,
                                  json=data,
                                  headers=REST_HEADERS)

        if response.status_code == requests.codes.ok:
            return JobRun.build(response.json())
        else:
            response.raise_for_status()

    def _change_jobrun_action_state(self, jobrun_id, jobrun_action_id, state, time=None):

        data = {"state": state}
        if time is not None:
            data["time"] = time.isoformat()

        url = self.baseurl + '/jobruns/' + jobrun_id + '/actions/' + jobrun_action_id
        response = requests.patch(url,
                                  json=data,
                                  headers=REST_HEADERS)

        if response.status_code == requests.codes.ok:
            return JobRunAction.build(response.json())
        else:
            response.raise_for_status()


class JobRun(object):
    """This class represents a jobrun."""

    def __init__(self, id_, job_id, username, state, start_time, end_time=None):
        self.id_ = id_
        self.job_id = job_id
        self.username = username
        self.state = state
        self.start_time = start_time
        self.end_time = end_time

    def __str__(self):
        return "{0} {1}".format(self.job_id, self.username)

    @staticmethod
    def build(json):
        """Build a JobRun object from JSON input."""
        end_time = None
        if json['end_time'] is not None:
            end_time = dateutil.parser.parse(json['end_time'])
        return JobRun(json['id'],
                      json['job_id'],
                      json['username'],
                      json['state'],
                      dateutil.parser.parse(json['start_time']),
                      end_time)


class JobRunAction(object):
    """This class represents a jobrun action."""

    def __init__(self, id_, jobrun_id, name, state, start_time, end_time=None):
        self.id_ = id_
        self.jobrun_id = jobrun_id
        self.name = name
        self.state = state
        self.start_time = start_time
        self.end_time = end_time

    def __str__(self):
        return "{0} {1} {2}".format(self.id_, self.jobrun_id, self.name)

    @staticmethod
    def build(json):
        """Build a JobRunAction object from JSON input."""
        end_time = None
        if json['end_time'] is not None:
            end_time = dateutil.parser.parse(json['end_time'])
        return JobRunAction(json['id'],
                            json['jobrun_id'],
                            json['name'],
                            json['state'],
                            dateutil.parser.parse(json['start_time']),
                            end_time)
